#include <iostream.h>
#include <stdlib.h>
#include <string.h>

/*
 * This program could more easily be accomplished with an if/else
 * set but this shows how functions can used as variables and
 * how structures group item types together
 */

// run with "varfunc methodX" where X is 1 thru 4

// prototype functions so we can use them in 'lookupTable'
bool method1(float, float);
bool method2(float, float);
bool method3(float, float);
bool method4(float, float);

// define array of structures to associate names and functions
struct {
  char* name;
  bool (*func)(float, float);
} lookupTable[] = {
  { "method1", method1 }, 
  { "method2", method2 },
  { "method3", method3 },
  { "method4", method4 },
  { 0, 0 }
};

/*
 * entry point
 */
int main(int argc, char* argv[])
{
  if (argc < 2) {
    cerr << "usage: " << argv[0] << " <method name>\n";
    exit(-1);
  }

  bool found = false;
  int i;

  for (i=0; lookupTable[i].name != 0; i++) {
    if (strcmp(argv[1], lookupTable[i].name) == 0) {
      if (lookupTable[i].func(1.0f, 2.0f)) {
        cout << lookupTable[i].name << " successful!\n";
      }
      else {
        cout << lookupTable[i].name << " failed\n";
      }
      found = true;
      break;
    }
  }

  if (!found) {
    cerr << argv[0] << ": unrecognised method; choices are:\n";
    for (i=0; lookupTable[i].name != 0; i++) {
      cerr << lookupTable[i].name << "\n";
    }
  }

  return 0;
}

/*
 * method1:
 */
bool method1(float a, float b)
{
  cout << "addition... ans = " << a+b << "\n";

  return true;
}

/*
 * method2:
 */
bool method2(float a, float b)
{
  cout << "subtraction... ans = " << a-b << "\n";

  return false; // for no good reason except as an example
}

/*
 * method3:
 */
bool method3(float a, float b)
{
  cout << "multiplication... ans = " << a*b  << "\n";

  return false;
}

/*
 * method4:
 */
bool method4(float a, float b)
{
  cout << "division... ans = " << a/b << "\n";

  return true;
}

